﻿function NodeViewModel(data) {
    var self = this;

    self.data = data;
    self.id = ecore.ko.observable(data.id);
    self.isExpanded = ecore.ko.observable(false);
    self.selected = ecore.ko.observable(data.selected || ecore.viewModel.findAsset(data.id));
    self.name = ecore.ko.observable(data.name);
    self.notTranslated = ecore.ko.observable(false);
    self.hasNodes = ecore.ko.observable(data.hasChildren);
    self.childrenLoaded = ecore.ko.observable(false);
    self.nodes = ecore.ko.observableArray([]);

    self.find = function (id) {
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }
        return null;
    };

    self.setTranslationStatus = function () {
        if (ecore.viewModel.languageFilter()) {
            self.notTranslated(self.data.missingLanguages.indexOf(ecore.viewModel.languageFilter()) >= 0);
            //console.log("setTranslationStatus: " + self.name() + " - " + self.notTranslated());
            return;
        }

        self.notTranslated(false);
        //console.log("setTranslationStatus: " + self.name() + " - " + self.notTranslated());
    }

    self.setTranslationStatus();

    self.setTranslationStatusTree = function () {
        var selfNodes = self.nodes();
        self.setTranslationStatus();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].setTranslationStatusTree();
    }

    self.addAllTree = function () {
        let opt = '';
        if (ecore.viewModel.syncBlocksFromPages() === 2) {
            opt = 'excludepagerefs';
        }
        else if (ecore.viewModel.syncBlocksFromPages() === 3) {
            opt = 'excluderefs';
        }
        ecore.viewModel.isLoading(true);
        var assets = ecore.contentStore.put({
            contentId: self.id(),
            id: ecore.getProjectId(),
            option: opt,
            action: 'fromallnode'
        });
        if (assets.then) {
            $n('#lion-btn-tree-save').attr('disabled', true);
            assets.then(function (item) {
                allMode = true;
                for (var i = 0; i < item.result.length; i++) {
                    if (item.result[i].type === "page") {
                        var asset = ecore.viewModel.findAsset(item.result[i].id);
                        if (!asset) {
                            ecore.viewModel.assets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = ecore.viewModel.findNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                    else if (item.result[i].type === "block") {
                        var blockAsset = ecore.viewModel.findBlockAsset(item.result[i].id);
                        if (!blockAsset) {
                            ecore.viewModel.blockAssets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = ecore.viewModel.findBlockNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                }
                allMode = false;
                $n('#lion-btn-tree-save').attr('disabled', false);
                ecore.viewModel.isLoading(false);
            });
        }
    };
    self.removeAllTree = function () {
        let opt = '';
        if (ecore.viewModel.syncBlocksFromPages() === 2) {
            opt = 'excludepagerefs';
        }
        else if (ecore.viewModel.syncBlocksFromPages() === 3) {
            opt = 'excluderefs';
        }
        ecore.viewModel.isLoading(true);
        var assets = ecore.contentStore.put({
            contentId: self.id(),
            id: ecore.getProjectId(),
            option: opt,
            action: 'fromallnode'
        });
        if (assets.then) {
            $n('#lion-btn-tree-save').attr('disabled', true);
            assets.then(function (item) {
                allMode = true;
                for (let i = 0; i < item.result.length; i++) {
                    if (item.result[i].type === "page") {
                        for (let k = 0; k < ecore.viewModel.assets().length; k++) {
                            if (ecore.viewModel.assets()[k].id !== item.result[i].id) continue;
                            let asset = ecore.viewModel.findAsset(item.result[i].id);
                            if (!asset) continue;
                            ecore.viewModel.assets.remove(asset);
                            --k;
                            var relatedNode = ecore.viewModel.findNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(false);
                            }
                        }
                    }
                    else if (item.result[i].type === "block") {
                        var blockAsset = ecore.viewModel.findBlockAsset(item.result[i].id);
                        if (blockAsset) {
                            for (let k = 0; k < ecore.viewModel.blockAssets().length; k++) {
                                if (ecore.viewModel.blockAssets()[k].id !== item.result[i].id) continue;
                                let asset = ecore.viewModel.findBlockAsset(item.result[i].id);
                                if (!asset) continue;
                                ecore.viewModel.blockAssets.remove(asset);
                                --k;
                                var relatedNode = ecore.viewModel.findBlockNode(item.result[i].id);
                                if (relatedNode != undefined && relatedNode != null) {
                                    relatedNode.selected(false);
                                }
                            }
                        }
                    }
                }
                $n('#lion-btn-tree-save').attr('disabled', false);
                allMode = false;
                ecore.viewModel.isLoading(false);
            });
        }
    };
    self.addOneAndRelatedNodes = function () {
        let opt = '';
        if (ecore.viewModel.syncBlocksFromPages() === 2) {
            opt = 'excludepagerefs';
        }
        else if (ecore.viewModel.syncBlocksFromPages() === 3) {
            opt = 'excluderefs';
        }
        ecore.viewModel.isLoading(true);
        var assets = ecore.contentStore.put({
            contentId: self.id(),
            id: ecore.getProjectId(),
            option: opt,
            action: 'fromonenode'
        });
        if (assets.then) {
            $n('#lion-btn-tree-save').attr('disabled', true);
            assets.then(function (item) {
                oneMode = true;
                for (var i = 0; i < item.result.length; i++) {
                    if (item.result[i].type === "page") {
                        var asset = ecore.viewModel.findAsset(item.result[i].id);
                        if (!asset) {
                            ecore.viewModel.assets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = ecore.viewModel.findNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                    else if (item.result[i].type === "block") {
                        var blockAsset = ecore.viewModel.findBlockAsset(item.result[i].id);
                        if (!blockAsset) {
                            ecore.viewModel.blockAssets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = ecore.viewModel.findBlockNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                }
                oneMode = false;
                $n('#lion-btn-tree-save').attr('disabled', false);
                ecore.viewModel.isLoading(false);
            });
        }
    };
    self.removeOneAndRelatedNodes = function () {
        let opt = '';
        if (ecore.viewModel.syncBlocksFromPages() === 2) {
            opt = 'excludepagerefs';
        }
        else if (ecore.viewModel.syncBlocksFromPages() === 3) {
            opt = 'excluderefs';
        }
        ecore.viewModel.isLoading(true);
        var assets = ecore.contentStore.put({
            contentId: self.id(),
            id: ecore.getProjectId(),
            option: opt,
            action: 'fromonenode'
        });
        if (assets.then) {
            $n('#lion-btn-tree-save').attr('disabled', true);
            assets.then(function (item) {
                allMode = true;
                for (let i = 0; i < item.result.length; i++) {
                    if (item.result[i].type === "page") {
                        for (let k = 0; k < ecore.viewModel.assets().length; k++) {
                            if (ecore.viewModel.assets()[k].id !== item.result[i].id) continue;
                            let asset = ecore.viewModel.findAsset(item.result[i].id);
                            if (!asset) continue;
                            ecore.viewModel.assets.remove(asset);
                            --k;
                            var relatedNode = ecore.viewModel.findNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(false);
                            }
                        }
                    }
                    else if (item.result[i].type === "block") {
                        var blockAsset = ecore.viewModel.findBlockAsset(item.result[i].id);
                        if (blockAsset) {
                            for (let k = 0; k < ecore.viewModel.blockAssets().length; k++) {
                                if (ecore.viewModel.blockAssets()[k].id !== item.result[i].id) continue;
                                let asset = ecore.viewModel.findBlockAsset(item.result[i].id);
                                if (!asset) continue;
                                ecore.viewModel.blockAssets.remove(asset);
                                --k;
                                var relatedNode = ecore.viewModel.findBlockNode(item.result[i].id);
                                if (relatedNode != undefined && relatedNode != null) {
                                    relatedNode.selected(false);
                                }
                            }
                        }
                    }
                }
                $n('#lion-btn-tree-save').attr('disabled', false);
                allMode = false;
                ecore.viewModel.isLoading(false);
            });
        }
    };
    
    self.selected.subscribe(function (value) {
        if (allMode || oneMode) return;
        if ($n('#lion-btn-tree-save'))
            $n('#lion-btn-tree-save').attr('disabled', true);
        if (value) {
            self.addOneAndRelatedNodes();
        }
        else
            self.removeOneAndRelatedNodes();
    });

    self.appendChildren = function (nodes) {
        self.childrenLoaded(true);

        for (var i = 0; i < nodes.length; i++)
            self.nodes.push(new NodeViewModel(nodes[i]));

    };

    if (data.nodes)
        self.appendChildren(data.nodes);

    self.toggleVisibility = function () {
        self.isExpanded(!self.isExpanded());

        if (self.isExpanded() && !self.childrenLoaded()) { // load new children
            self.childrenLoaded(true);
            ecore.loadChildNodes(self.id(), self);
        }
    };
};

var allMode = false;
var oneMode = false;

function BlockNodeViewModel(data) {
    var self = this;

    self.data = data;
    self.id = ecore.ko.observable(data.id);
    self.isExpanded = ecore.ko.observable(false);
    self.selected = ecore.ko.observable(data.selected || ecore.viewModel.findBlockAsset(data.id));
    self.selectable = ecore.ko.observable(data.selectable);
    self.name = ecore.ko.observable(data.name);
    self.notTranslated = ecore.ko.observable(false);
    self.hasNodes = ecore.ko.observable(data.hasChildren);
    self.childrenLoaded = ecore.ko.observable(false);
    self.blockNodes = ecore.ko.observableArray([]);

    self.find = function (id) {
        var selfNodes = self.blockNodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }
        return null;
    };

    self.setTranslationStatus = function () {
        if (ecore.viewModel.languageFilter()) {
            self.notTranslated(self.data.missingLanguages.indexOf(ecore.viewModel.languageFilter()) >= 0);
            //console.log("setTranslationStatus: " + self.name() + " - " + self.notTranslated());
            return;
        }
        self.notTranslated(false);
        //console.log("setTranslationStatus: " + self.name() + " - " + self.notTranslated());
    }

    self.setTranslationStatus();

    self.setTranslationStatusTree = function () {
        var selfNodes = self.blockNodes();
        self.setTranslationStatus();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].setTranslationStatusTree();
    }

    self.addAllTree = function () {
        let opt = '';
        if (ecore.viewModel.syncNestedPageBlocksFromSelectedBlock()) {
            opt = 'excluderefs';
        }
        ecore.viewModel.isLoading(true);
        var assets = ecore.contentStore.put({
            contentId: self.id(),
            id: ecore.getProjectId(),
            option: opt
        });
        if (assets.then) {
            $n('#lion-btn-tree-save').attr('disabled', true);
            assets.then(function (item) {
                allMode = true;
                for (var i = 0; i < item.result.length; i++) {
                    if (item.result[i].type === "page") {
                        var asset = ecore.viewModel.findAsset(item.result[i].id);
                        if (!asset) {
                            ecore.viewModel.assets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = ecore.viewModel.findNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                    else if (item.result[i].type === "block") {
                        var blockAsset = ecore.viewModel.findBlockAsset(item.result[i].id);
                        if (!blockAsset) {
                            ecore.viewModel.blockAssets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = ecore.viewModel.findBlockNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                }
                $n('#lion-btn-tree-save').attr('disabled', false);
                ecore.viewModel.isLoading(false);
                allMode = false;
            });
        }
    };
    self.removeAllTree = function () {
        let opt = '';
        if (ecore.viewModel.syncNestedPageBlocksFromSelectedBlock()) {
            opt = 'excluderefs';
        }
        ecore.viewModel.isLoading(true);
        var assets = ecore.contentStore.put({
            contentId: self.id(),
            id: ecore.getProjectId(),
            option: opt
        });
        if (assets.then) {
            $n('#lion-btn-tree-save').attr('disabled', true);
            assets.then(function (item) {
                allMode = true;
                for (var i = 0; i < item.result.length; i++) {
                    if (item.result[i].type === "page") {
                        for (let k = 0; k < ecore.viewModel.assets().length; k++) {
                            if (ecore.viewModel.assets()[k].id !== item.result[i].id) continue;
                            let asset = ecore.viewModel.findAsset(item.result[i].id);
                            if (!asset) continue;
                            ecore.viewModel.assets.remove(asset);
                            --k;
                            var relatedNode = ecore.viewModel.findNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(false);
                            }
                        }

                    }
                    else if (item.result[i].type === "block") {
                        var blockAsset = ecore.viewModel.findBlockAsset(item.result[i].id);
                        if (blockAsset) {
                            for (let k = 0; k < ecore.viewModel.blockAssets().length; k++) {
                                if (ecore.viewModel.blockAssets()[k].id !== item.result[i].id) continue;
                                let asset = ecore.viewModel.findBlockAsset(item.result[i].id);
                                if (!asset) continue;
                                ecore.viewModel.blockAssets.remove(asset);
                                --k;
                                var relatedNode = ecore.viewModel.findBlockNode(item.result[i].id);
                                if (relatedNode != undefined && relatedNode != null) {
                                    relatedNode.selected(false);
                                }
                            }

                        }
                    }
                }
                allMode = false;
                $n('#lion-btn-tree-save').attr('disabled', false);
                ecore.viewModel.isLoading(false);
            });
        }
    };
    self.addOneAndRelatedNodes = function () {
        let opt = '';
        if (ecore.viewModel.syncNestedPageBlocksFromSelectedBlock()) {
            opt = 'excluderefs';
        }
        ecore.viewModel.isLoading(true);
        var assets = ecore.contentStore.put({
            contentId: self.id(),
            id: ecore.getProjectId(),
            option: opt,
            action: 'fromonenode'
        });
        if (assets.then) {
            $n('#lion-btn-tree-save').attr('disabled', true);
            assets.then(function (item) {
                oneMode = true;
                for (var i = 0; i < item.result.length; i++) {
                    if (item.result[i].type === "page") {
                        var asset = ecore.viewModel.findAsset(item.result[i].id);
                        if (!asset) {
                            ecore.viewModel.assets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = ecore.viewModel.findNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                    else if (item.result[i].type === "block") {
                        var blockAsset = ecore.viewModel.findBlockAsset(item.result[i].id);
                        if (!blockAsset) {
                            ecore.viewModel.blockAssets.push({
                                id: item.result[i].id,
                                name: item.result[i].name,
                                status: '',
                                isDeleted: false,
                                referers: [],
                                isNotObservable: true
                            });

                            var relatedNode = ecore.viewModel.findBlockNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(true);
                            }
                        }
                    }
                }
                $n('#lion-btn-tree-save').attr('disabled', false);
                ecore.viewModel.isLoading(false);
                oneMode = false;
            });
        }
    };
    self.removeOneAndRelatedNodes = function () {
        let opt = '';
        if (ecore.viewModel.syncNestedPageBlocksFromSelectedBlock()) {
            opt = 'excluderefs';
        }
        ecore.viewModel.isLoading(true);
        var assets = ecore.contentStore.put({
            contentId: self.id(),
            id: ecore.getProjectId(),
            option: opt,
            action: 'fromonenode'
        });
        if (assets.then) {
            $n('#lion-btn-tree-save').attr('disabled', true);
            assets.then(function (item) {
                allMode = true;
                for (var i = 0; i < item.result.length; i++) {
                    if (item.result[i].type === "page") {
                        for (let k = 0; k < ecore.viewModel.assets().length; k++) {
                            if (ecore.viewModel.assets()[k].id !== item.result[i].id) continue;
                            let asset = ecore.viewModel.findAsset(item.result[i].id);
                            if (!asset) continue;
                            ecore.viewModel.assets.remove(asset);
                            --k;
                            var relatedNode = ecore.viewModel.findNode(item.result[i].id);
                            if (relatedNode != undefined && relatedNode != null) {
                                relatedNode.selected(false);
                            }
                        }

                    }
                    else if (item.result[i].type === "block") {
                        var blockAsset = ecore.viewModel.findBlockAsset(item.result[i].id);
                        if (blockAsset) {
                            for (let k = 0; k < ecore.viewModel.blockAssets().length; k++) {
                                if (ecore.viewModel.blockAssets()[k].id !== item.result[i].id) continue;
                                let asset = ecore.viewModel.findBlockAsset(item.result[i].id);
                                if (!asset) continue;
                                ecore.viewModel.blockAssets.remove(asset);
                                --k;
                                var relatedNode = ecore.viewModel.findBlockNode(item.result[i].id);
                                if (relatedNode != undefined && relatedNode != null) {
                                    relatedNode.selected(false);
                                }
                            }

                        }
                    }
                }
                allMode = false;
                $n('#lion-btn-tree-save').attr('disabled', false);
                ecore.viewModel.isLoading(false);
            });
        }
    };
    
    self.selected.subscribe(function (value) {
        if (allMode || oneMode) return;
        if ($n('#lion-btn-tree-save'))
            $n('#lion-btn-tree-save').attr('disabled', true);
        if (value) {
            self.addOneAndRelatedNodes();
        }
        else
            self.removeOneAndRelatedNodes();
    });

    self.appendBlockChildren = function (blockNodes) {
        self.childrenLoaded(true);
        for (var i = 0; i < blockNodes.length; i++)
            self.blockNodes.push(new BlockNodeViewModel(blockNodes[i]));
    };

    self.toggleVisibility = function () {
        self.isExpanded(!self.isExpanded());

        if (self.isExpanded() && !self.childrenLoaded()) { // load new children
            self.childrenLoaded(true);
            ecore.loadChildBlockNodes(self.id(), self);
        }
    };

    //initialization of data
    if (data.blockNodes)
        self.appendBlockChildren(data.blockNodes);

};

function AssetViewModel() {
    var self = this;
    self.isLoading = ecore.ko.observable(false);
    self.isSaving = ecore.ko.observable(false);
    self.progress = ecore.ko.observable(0);
    self.progressTotal = ecore.ko.observable(0);
    self.assets = ecore.ko.observableArray([]);
    self.nodes = ecore.ko.observableArray([]);
    self.blockAssets = ecore.ko.observableArray([]);
    self.blockNodes = ecore.ko.observableArray([]);
    self.editable = ecore.ko.observable(false);
    self.syncBlocksFromPages = ecore.ko.observable(1);
    self.syncNestedPageBlocksFromSelectedBlock = ecore.ko.observable(true);


    self.languageFilter = ecore.ko.observable('');
    self.availableLanguages = ecore.ko.observableArray([]);
    self.search = ecore.ko.observable('');
    self.throttledSearch = ecore.ko.computed(self.search).extend({ throttle: 400 });
    self.searchResults = ecore.ko.observableArray([]);
    self.showSearchPane = ecore.ko.observable(false);

    self.languageFilter.subscribe(function (newValue) {
        var length = self.nodes().length;
        for (var i = 0; i < length; i++) {
            self.nodes()[i].setTranslationStatusTree();
        }
        length = self.blockNodes().length;
        for (var i = 0; i < length; i++) {
            self.blockNodes()[i].setTranslationStatusTree();
        }
    });

    self.findAsset = function (id) {
        var selfAssets = self.assets();
        for (var i = 0; i < selfAssets.length; i++) {
            if (selfAssets[i].id == id)
                return selfAssets[i];
        }
        return null;
    };

    self.findBlockAsset = function (id) {
        var selfBlockAssets = self.blockAssets();
        for (var i = 0; i < selfBlockAssets.length; i++) {
            if (selfBlockAssets[i].id == id)
                return selfBlockAssets[i];
        }
        return null;
    };       

    self.findNode = function (id) {
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }
        return null;
    }; 

    self.findBlockNode = function (id) {
        var selfNodes = self.blockNodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }
        return null;
    };

    self.deselectBlockAssetInTree = function (blockAsset) {
        self.blockAssets.remove(blockAsset);

        var blockNode = self.findBlockNode(blockAsset.id);
        if (blockNode)
            blockNode.selected(false);
        //else
        
    };

    self.deselectInTree = function (asset) {
        self.assets.remove(asset);

        var node = self.findNode(asset.id);
        if (node)
            node.selected(false);
    };
   
    //do the search
    self.throttledSearch.subscribe(function (newValue) {
        if (newValue.length <= 1) return;

        //console.log("Do Search for: " + newValue);

        ecore.search(newValue);
    });      

    self.save = function () {
        $n('#lion-btn-tree-save').attr('disabled', true);
        ecore.saveAssets();
        
    };

    self.close = function () {
        ecore.closeDialog();
    };

    self.appendChildren = function (nodes) {
        for (var i = 0; i < nodes.length; i++)
            self.nodes.push(new NodeViewModel(nodes[i]));
    };

    self.appendSearchResult = function (nodes) {
        self.searchResults([]);

        for (var i = 0; i < nodes.length; i++)
            self.searchResults.push(new NodeViewModel(nodes[i]));

        self.showSearchPane(true);
    };

    self.hideSearch = function (event) {
        var e = event || window.event;
        if (e.tag === "click_in_search_pane") return;

        self.showSearchPane(false);
    };

    self.preventDefault = function (event) {
        var e = event || window.event;
        e.tag = "click_in_search_pane";
    };

    self.appendBlockChildren = function (blockNodes) {
        for (var i = 0; i < blockNodes.length; i++)
            self.blockNodes.push(new BlockNodeViewModel(blockNodes[i]));
    };
}

define([
    "dojo/dom",
    "dojo/dom-construct",
    "dijit/form/Select",
    "dojo/_base/window",
    "dojo/domReady!"
], function (
    dom,
    domConstruct,
    Select,
    win
) {
    window["ecore"] = {
        epieditor: null,
        dataStore: null,
        treeStore: null,
        treeBlockStore: null,
        searchStore: null,
        viewModel: null,
        ko: null,
        _projectId: -1,
        dialog: null,
        inited: false,

        //registry.get("workspace.translationprojecteditorstore"),
        //registry.get("workspace.translationprojecttreestore"),
        //registry.get("workspace.translationsearchitemsstore"),

        init: function (epieditor, dataStore, treeStore, treeBlockStore, searchStore, contentStore, ko) {
            var self = this;
            self.dataStore = dataStore;
            self.epieditor = epieditor;
            self.treeStore = treeStore;
            self.treeBlockStore = treeBlockStore;
            self.searchStore = searchStore;
            self.contentStore = contentStore;
            
            self.ko = ko;

            if (!self.dialog)
                self.dialog = new dijit.Dialog({ content: dojo.cache("workspace.editors", "multiselecteditorlightbox.html?v=" + new Date().getTime()), style: 'width: 90%; height: 95%', title: 'Assets in Project' });

            if (!self.inited) {
                try {
                    self.initTabs();
                    self.createNewModel();
                    self.bindModel();
                    self.inited = true;

                    var nitecoSelect = new Select({
                        name: "select2",
                        options: [
                            { label: "Include nested pages & blocks", value: 1, selected: true },
                            { label: "Include blocks", value: 2 },
                            { label: "Exclude nested pages & blocks", value: 3 }                                                     
                        ]
                    });
                    nitecoSelect.placeAt(dom.byId('niteco-epi-lb-right-tree'));
                    nitecoSelect.on("change", function () {
                        //console.log("my value: ", this.get("value"));
                        ecore.viewModel.syncBlocksFromPages = ecore.ko.observable(this.get("value"));
                    })
                } catch (err) {
                    console.error(err.message);
                }
            }
           
        },

        getProjectId: function () {
            if (this._projectId == -1)
                this._projectId = $n('#lb-project-id').val();

            return this._projectId;
        },

        createNewModel: function () {
            this.viewModel = new AssetViewModel();
        },

        showDialog: function () {
            var self = this;
            self.emptyModel();
            self.loadAssets();
            self.loadRootNodes();
            self.loadRootBlockNodes();
            self.dialog.show();
            self.loadDialog(this.dialog);

            //console.log('Loading project: ' + self.getProjectId());
        },

        loadRootNodes: function () {
            this.viewModel.nodes([]);
            this.loadChildNodes(-1, this.viewModel);
        },

        loadChildNodes: function (parentID, model) {
            var self = this;
            var parameters = parentID + ";" + ecore.getProjectId();
            if (self.viewModel.languageFilter()) {
                parameters += ";" + self.viewModel.languageFilter();
            }
            var query = self.treeStore.get(parameters + "?" + Math.random() * 9999999999);

            if (query.then) {
                query.then(function (item) {

                    var assets = self.viewModel.assets();
                    for (var i = 0; i < item.result.length; i++) {
                        var isSelectedAsset = false;
                        for (var j = 0; j < assets.length; j++) {

                            if (item.result[i].id == assets[j].id) {
                                item.result.selected = true;
                                isSelectedAsset = true;
                                break;
                            }
                        }

                        if (!isSelectedAsset) {
                            item.result.selected = false;
                        }
                    }
                    model.appendChildren(item.result);

                    //var test = self.viewModel.assets();
                });
            } else {
                model.appendChildren(query.result);
            }
        },

        loadRootBlockNodes: function () {
            this.viewModel.blockNodes([]);
            this.loadChildBlockNodes(-1, this.viewModel);
        },

        loadChildBlockNodes: function (parentID, model) {
            var self = this;
            var parameters = parentID + ";" + ecore.getProjectId();
            if (self.viewModel.languageFilter()) {
                parameters += ";" + self.viewModel.languageFilter();
            }

            var query = self.treeBlockStore.get(parameters + "?" + Math.random() * 9999999999);

            if (query.then) {
                query.then(function (item) {

                    var blockAssets = self.viewModel.blockAssets();
                    for (var i = 0; i < item.result.length; i++) {
                        var isSelectedAsset = false;
                        for (var j = 0; j < blockAssets.length; j++) {

                            if (item.result[i].id == blockAssets[j].id) {
                                item.result.selected = true;
                                isSelectedAsset = true;
                                break;
                            }
                        }

                        if (!isSelectedAsset) {
                            item.result.selected = false;
                        }
                    }

                    model.appendBlockChildren(item.result);
                });
            } else {
                model.appendBlockChildren(query.result);
            }

        },

        saveAssets: function () {
            var self = this;
            var data = self.getProjectId();
            var assets = self.viewModel.assets();

            for (var i = 0; i < assets.length; i++)
                data += '_' + assets[i].id;

            var blockAssets = self.viewModel.blockAssets();
            for (var i = 0; i < blockAssets.length; i++)
                data += '_' + blockAssets[i].id;

            ecore.viewModel.isSaving(true);
            self.dataStore.put(data).then(function (response) {
                $n('#lion-btn-tree-save').attr('disabled', false);
                ecore.closeDialog();
                ecore.viewModel.isSaving(false);
            });
        },

        closeDialog: function () {
            this.dialog.hide();
        },

        emptyModel: function () {
            this._projectId = -1;
            this.viewModel.assets([]);
            this.viewModel.nodes([]);
            this.viewModel.search('');
            this.viewModel.languageFilter('');
        },

        bindModel: function () {
            this.ko.applyBindings(this.viewModel, document.getElementById("lb-editor"));
        },

        unbindModel: function () {
            var self = this;
            var node = $n("#lb-editor");
            node.find("*").each(function () {
                $n(this).unbind();
            });

            ko.cleanNode(document.getElementById("lb-editor"));
        },

        getHeight: function () {
            var total = $n('.dijitDialog').outerHeight();
            var title = $n('.dijitDialogTitleBar').outerHeight();
            return total - title;
        },

        loadDialog: function (dialog) {
            var self = this;
            var size = self.getHeight();
            $n('.dijitDialogPaneContent').css('height', size + 'px');
            $n('.dijitDialogPaneContent').css('padding', '0px');
            $n('.lb-pane').css('max-height', (size - 145) + 'px');
            $n('.lb-pane').css('height', (size - 145) + 'px');
            self.dialog = dialog;
        },

        initTabs: function () {
            var self = this;
            var size = self.getHeight() - 145;

            var tc = new dijit.layout.TabContainer({
                style: "width: 100%; height: " + size + "px;",
                useMenu: false,
                useSlider: false,
                doLayout: false
            }, "lb-editor-tabContainer");


            var cp1 = new dijit.layout.ContentPane({
                title: "Pages",
                content: dojo.cache("workspace.editors", "multiselecteditorlightboxpages.html?v=" + new Date().getTime()),
                style: "width:100%; height: 100%;",
                selected: true,
                doLayout: false
            });
            tc.addChild(cp1);

            var cp2 = new dijit.layout.ContentPane({
                title: "Blocks",
                content: dojo.cache("workspace.editors", "multiselecteditorlightboxblocks.html?v=" + new Date().getTime()),
                style: "width:100%; height: 100%;",
                doLayout: false
            });
            tc.addChild(cp2);

            tc.startup();
        },

        loadAssets: function () {
            var self = this;
            var query = self.dataStore.get(self.getProjectId() + "?" + Math.random() * 9999999999);

            if (!query.then) return;

            query.then(function (item) {
                self.viewModel.editable(item.result.editable);
                self.viewModel.assets(item.result.assets);
                self.viewModel.blockAssets(item.result.blockAssets);
                self.viewModel.availableLanguages(item.result.availableLanguages);
            });
        },

        search: function (searchString) {
            var self = this;
            var query = self.searchStore.get(self.getProjectId() + ";" + searchString + "?" + Math.random() * 9999999999);

            if (query.then) {
                query.then(function (item) {
                    self.viewModel.appendSearchResult(item.result);
                });
            }
            else {
                self.viewModel.appendSearchResult(query.result);
            }
        }
    };
});



