﻿define([
    "dojo",
    "dojo/query",
    "dojo/_base/declare",
    "dijit/_Widget",
    "dijit/_TemplatedMixin",
    "epi/dependency",
    "epi/shell/widget/dialog/Confirmation",
    "epi/shell/widget/dialog/Alert"
    //custom
], function (
    dojo,
    query,
    declare,
    _Widget,
    _TemplatedMixin,
    dependency,
    Confirmation,
    Alert
    //custom
) {
    return declare("workspace.editors.sendjobeditor.sendjobeditor", [_Widget, _TemplatedMixin], {
        // templateString: [protected] String  
        //    A string that represents the default widget template.  
        //templateString: dojo.cache("workspace.editors", "MultiSelectEditor.template"),
        templateString: dojo.cache("workspace.editors.sendjobeditor", "sendjobeditor.html"),
        dialog: null,

        postCreate: function () {
            if (this.readOnly) {
                this.domNode.style.pointerEvents = 'none';
                this.domNode.style.opacity = 0.4;
            }
        },

        focus: function () {
            // summary:  
            //    Put focus on this widget.  
            // tags:  
            //    public  

            dijit.focus(this.selected);
        },

        viewModel: {
            //assets: ko.observableArray()
        },

        isValid: function () {
            // summary:  
            //    Indicates whether the current value is valid.  
            // tags:  
            //    public  

            return true;
        },

        onChange: function (value) {
            // summary:   
            //    Called when the value in the widget changes.   
            // tags:   
            //    public callback   
        },


        _onIntermediateChange: function (event) {
            // summary:   
            //    Handles the textbox key press events event and populates this to the onChange method.   
            // tags:   
            //    private   

            if (this.intermediateChanges) {
                this._set('value', event.target.value);
                this.onChange(this.value);
            }
        },

        _sendJob: function (event) {
            event.cancelBubble = true;
            var self = this;
            var lionConfirmation = new Confirmation({
                title: "Confirm Sending",
                description: "Are you sure you want to send?",
                onAction: function (confirmed) {
                    if (confirmed) {
                        setTimeout(function () {
                            var registry = dependency.resolve("epi.storeregistry");
                            var store = registry.get("workspace.lionjobstore");

                            //var store = registry.get("workspace.translationprojectstore");
                            store.put({
                                Id: self.value,
                                Action: 'send'
                            }).then(function (response) {
                                console.log("sending success");
                                if (response.error) {
                                    var errorDialog = new Alert({
                                        heading: "Job not sent",
                                        description: "Something went wrong when sending the job due to: " + response.errorMessage,
                                        onAction: function () {
                                            if (typeof LionBridgeConnectorGrid !== "undefined") {
                                                LionBridgeConnectorGrid.onReloadGrid();
                                            }
                                            //Refresh gadget also
                                            if (typeof LionBridgeConnectorGadget !== "undefined") {
                                                LionBridgeConnectorGadget.reloadGadget();
                                            }
                                        }
                                    });
                                    errorDialog.show();
                                } else if ('case did send') {
                                    var dialog = new Alert({
                                        heading: "Job sending",
                                        description:
                                            "The job is sending... The status will update when processing is complete",
                                        onAction: function () {
                                            location.reload();
                                            if (typeof LionBridgeConnectorGrid !== "undefined") {
                                                LionBridgeConnectorGrid.onReloadGrid();
                                            }
                                            //Refresh gadget also
                                            if (typeof LionBridgeConnectorGadget !== "undefined") {
                                                LionBridgeConnectorGadget.reloadGadget();
                                            }
                                        }
                                    });

                                    dialog.show();
                                } else {
                                    if (typeof LionBridgeConnectorGrid !== "undefined") {
                                        LionBridgeConnectorGrid.onReloadGrid();
                                    }
                                    //Refresh gadget also
                                    if (typeof LionBridgeConnectorGadget !== "undefined") {
                                        LionBridgeConnectorGadget.reloadGadget();
                                    }
                                }
                            });
                        }, 1);
                    }
                }
            });

            lionConfirmation.show();
        },

        _onChange: function (event) {
            // summary:   
            //    Handles the textbox change event and populates this to the onChange method.   
            // tags:   
            //    private   

            this._set('value', event.target.value);
            this.onChange(this.value);
        },

        _setValueAttr: function (value) {
            // summary:   
            //    Sets the value of the widget to "value" and updates the value displayed in the textbox.   
            // tags:   
            //    private   

            this._set('value', value);
            this.selected.value = this.value || '';
        }
    });
});